// ignore_for_file: public_member_api_docs

import 'dart:math';

import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:fl_chart/fl_chart.dart';
import 'package:flutter/material.dart';

part 'channel.g.dart';

/// [Channel] is a widget drawing an ecg sequence per channel.
@CopyWith()
class Channel extends StatelessWidget {
  /// [Channel] is a widget that draw a graph of
  /// an ecg sequence by a given [ecgSequence].
  const Channel({
    super.key,
    required this.ecgSequence,
    this.samplingRate = 250,
    this.strokeWidth = 1.5,
    this.color = Colors.black,
    this.isFiltered = true,
    this.isVisibleGrid = false,
    this.measureTime = 60,
    this.maxY = 2,
    this.minY = -2,
  });

  final int measureTime;
  final bool isFiltered;
  final double strokeWidth;
  final int samplingRate;
  final List<FlSpot> ecgSequence;
  final bool isVisibleGrid;

  final double maxY;
  final double minY;

  final Color color;

  @override
  Widget build(BuildContext context) {
    final ecg = ecgSequence.sublist(
      max(0, ecgSequence.length - samplingRate * measureTime),
      ecgSequence.length,
    );

    if(ecg.isEmpty) return const SizedBox();

    final minX = ecg.first.x;
    final maxX = ecg.first.x + samplingRate * measureTime.toDouble();

    return LineChart(
      LineChartData(
        minX: minX,
        maxX: maxX,
        minY: isFiltered ? minY : null,
        maxY: isFiltered ? maxY : null,
        backgroundColor: Colors.transparent,
        lineTouchData: LineTouchData(enabled: false),
        gridData: FlGridData(
          show: false,
        ),
        titlesData: FlTitlesData(
          show: false,
        ),
        borderData: FlBorderData(
          show: false,
        ),
        lineBarsData: [
          if (isFiltered && isVisibleGrid)
            for (double i = 0; i <= maxX; i += samplingRate / 5)
              LineChartBarData(
                spots: [
                  FlSpot(i, minY),
                  FlSpot(i, maxY),
                ],
                isCurved: false,
                color: Colors.blueGrey,
                barWidth: 0.3,
                dotData: FlDotData(
                  show: false,
                ),
              ),
          if (isFiltered && isVisibleGrid)
            for (double i = 0; i <= maxX; i += samplingRate / 25)
              LineChartBarData(
                spots: [
                  FlSpot(i, minY),
                  FlSpot(i, maxY),
                ],
                isCurved: false,
                color: Colors.blueGrey,
                barWidth: 0.1,
                dotData: FlDotData(
                  show: false,
                ),
              ),
          if (isFiltered && isVisibleGrid)
            for (double i = minY; i <= maxY; i += 1)
              LineChartBarData(
                spots: [
                  FlSpot(minX, i),
                  FlSpot(maxX, i),
                ],
                isCurved: false,
                color: Colors.blueGrey,
                barWidth: 0.3,
                dotData: FlDotData(
                  show: false,
                ),
              ),
          if (isFiltered && isVisibleGrid)
            for (double i = minY; i <= maxY; i += 1 / 5)
              LineChartBarData(
                spots: [
                  FlSpot(minX, i),
                  FlSpot(maxX, i),
                ],
                isCurved: false,
                color: Colors.blueGrey,
                barWidth: 0.1,
                dotData: FlDotData(
                  show: false,
                ),
              ),
          LineChartBarData(
            spots: ecg,
            color: color,
            isCurved: false,
            barWidth: strokeWidth,
            dotData: FlDotData(
              show: false,
            ),
          ),
        ],
      ),
      chartRendererKey: UniqueKey(),
      swapAnimationDuration: Duration.zero, // Optional
    );
  }
}
